/*
 * The Clear BSD License
 * Copyright 2017 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted (subject to the limitations in the disclaimer below) provided
 *  that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY THIS LICENSE.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _BOARD_H_
#define _BOARD_H_

#include "clock_config.h"
#include "fsl_common.h"
#include "fsl_gpio.h"

/*******************************************************************************
 * Definitions
 ******************************************************************************/
/*! @brief The board name */
#define BOARD_NAME "MIMXRT1020-EVK"

/* The UART to use for debug messages. */
#define BOARD_DEBUG_UART_TYPE DEBUG_CONSOLE_DEVICE_TYPE_LPUART
#define BOARD_DEBUG_UART_BASEADDR (uint32_t) LPUART1
#define BOARD_DEBUG_UART_CLK_FREQ BOARD_DebugConsoleSrcFreq()
#define BOARD_UART_IRQ LPUART1_IRQn
#define BOARD_UART_IRQ_HANDLER LPUART1_IRQHandler

#ifndef BOARD_DEBUG_UART_BAUDRATE
#define BOARD_DEBUG_UART_BAUDRATE 115200
#endif

/* FlexIO Camera configurations */
#define BOARD_CAMERA_FLEXIO_INST                    FLEXIO1
#define BOARD_CAMERA_FLEXIO_DATA_PIN_START_INDEX    24
#define BOARD_CAMERA_FLEXIO_DATA_PIN_END_INDEX      31
#define BOARD_CAMERA_FLEXIO_PCLK_PIN_INDEX          16
#define BOARD_CAMERA_FLEXIO_HREF_PIN_INDEX          18
#define BOARD_CAMERA_VSYNC_GPIO_BASE                GPIO2
#define BOARD_CAMERA_VSYNC_PIN_INDEX                5u
#define BOARD_CAMERA_VSYNC_IRQn                     GPIO2_Combined_0_15_IRQn
#define BOARD_CAMERA_VSYNC_IRQHandler               GPIO2_Combined_0_15_IRQHandler
#define BOARD_CAMERA_RST_GPIO_BASE                  GPIO2
#define BOARD_CAMERA_RST_PIN_INDEX                  7u
#define BOARD_CAMERA_PDWN_GPIO_BASE                 GPIO2
#define BOARD_CAMERA_PDWN_PIN_INDEX                 16u

/* I2C for camera SCCB. */
#define BOARD_CAMERA_I2C_INST                       LPI2C2

/* I2C for camera SCCB. */
#define BOARD_I2C_SCL_GPIO_BASE         GPIO2
#define BOARD_I2C_SDA_GPIO_BASE         GPIO2
#define BOARD_I2C_SCL_PIN_INDEX         19u
#define BOARD_I2C_SDA_PIN_INDEX         18u

/* GPIO for SPI LCD pins. */
#define BOARD_SPILCD_SPI_INST           LPSPI3
#define BOARD_SPILCD_RST_GPIO_BASE      GPIO3
#define BOARD_SPILCD_RST_PIN_INDEX      21u
#define BOARD_SPILCD_DC_GPIO_BASE       GPIO3
#define BOARD_SPILCD_DC_PIN_INDEX       22u
#define BOARD_SPILCD_CS_GPIO_BASE       GPIO3
#define BOARD_SPILCD_CS_PIN_INDEX       20u

/* DMA channel assigements. */
#define FLEXIO_CAMERA_DMA_CHN           0u
#define FLEXIO_CAMERA_DMA_MUX_SRC       (kDmaRequestMuxFlexIO1Request0Request1 & 0xFF)

#define SPI_LCD_DMA_CHN                 1u
#define SPI_LCD_DMA_MUX_SRC             (kDmaRequestMuxLPSPI3Tx & 0xFF)
#define SPI_LCD_DMA_IRQn                DMA1_DMA17_IRQn
#define SPI_LCD_DMA_IRQHandler          DMA1_DMA17_IRQHandler


#define BOARD_I2C_EMU_WITH_GPIO         true

#if defined(__cplusplus)
extern "C" {
#endif /* __cplusplus */

/*******************************************************************************
 * API
 ******************************************************************************/
uint32_t BOARD_DebugConsoleSrcFreq(void);

void BOARD_InitDebugConsole(void);
void BOARD_ConfigMPU(void);
void BOARD_SPILCD_RstPinInit(void);
void BOARD_SPILCD_DcPinInit(void);
void BOARD_SPILCD_CsPinInit(void);
void BOARD_SPILCD_pullRstPin(bool value);
void BOARD_SPILCD_pullDcPin(bool value);
void BOARD_SPILCD_pullCsPin(bool value);
void BOARD_Camera_RstPinInit(void);
void BOARD_Camera_PdwnPinInit(void);
void BOARD_Camera_VsynPinInit(void);
void BOARD_Camera_pullRstPin(bool value);
void BOARD_Camera_pullPdwnPin(bool value);

#if defined(__cplusplus)
}
#endif /* __cplusplus */

#endif /* _BOARD_H_ */
